'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.isFormValid = isFormValid;
exports.validateDataAgainstSchema = validateDataAgainstSchema;
exports.validateSingleField = validateSingleField;
exports.validateObject = validateObject;

var _lodash = require('lodash');

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _dataTypes = require('../data-types');

var _dataTypes2 = _interopRequireDefault(_dataTypes);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createError = function createError(error) {
  return { result: false, error: error };
};

var isLessThanMinValue = function isLessThanMinValue(value, minValue) {
  if ((0, _util.isNullOrUndefined)(minValue)) {
    return false;
  }

  if ((0, _util.isNullOrUndefined)(value)) {
    return true;
  } else if (value < minValue) {
    return true;
  }
  return false;
};

var isGreaterThanMaxValue = function isGreaterThanMaxValue(value, maxValue) {
  if ((0, _util.isNullOrUndefined)(maxValue)) {
    return false;
  }

  if ((0, _util.isNullOrUndefined)(value)) {
    return true;
  } else if (value > maxValue) {
    return true;
  }
  return false;
};

var isValid = function isValid(text, validatorToUse) {
  if (!validatorToUse) {
    return true;
  }

  if (!(0, _lodash.isEmpty)(text) && !_validator2.default[validatorToUse]('' + text)) {
    return false;
  }
  return true;
};

var isValidRegex = function isValidRegex(text, validationRegex) {
  if ((0, _util.isNullOrUndefined)(text) || (0, _util.isNullOrUndefined)(validationRegex)) {
    return true;
  }
  var pattern = new RegExp(validationRegex);
  var response = text.match(pattern);
  if (!response) {
    return false;
  }
  return true;
};

function isFormValid(form) {
  var formFields = (0, _lodash.omit)(form, 'schema', 'valid');
  var valid = true;
  (0, _lodash.forOwn)(formFields, function (value) {
    valid = valid && (value && value.validationResult ? value.validationResult.result : false);
  });
  return valid;
}

function validateDataAgainstSchema(name, value, schema) {
  if ((0, _util.isNullOrUndefined)(schema)) {
    return { result: true };
  }

  switch ((0, _lodash.toUpper)(schema.fieldType)) {
    case _dataTypes2.default.Any:
      if (schema.isRequired && ((0, _util.isNullOrUndefined)(value) || (0, _lodash.isEmpty)(value))) {
        return {
          result: false,
          error: '*Required'
        };
      }
      return { result: true };
    case _dataTypes2.default.String:
      if (schema.isRequired && ((0, _util.isNullOrUndefined)(value) || (0, _lodash.isEmpty)(value))) {
        return {
          result: false,
          error: '*Required'
        };
      }

      if (!isValid(value, schema.validatorType)) {
        return createError('*Invalid value');
      }

      if (!isValidRegex(value, schema.validationRegex)) {
        return createError('*Invalid value');
      }
      return { result: true };
    case _dataTypes2.default.Number:
      if (schema.isRequired) {
        if ((0, _util.isNullOrUndefined)(value)) {
          return {
            result: false,
            error: '*Required'
          };
        }

        if (typeof value === 'string') {
          if ((0, _lodash.isEmpty)(value)) {
            return {
              result: false,
              error: '*Required'
            };
          }

          var numberVal = (0, _lodash.toNumber)(value);
          if (!(0, _lodash.isFinite)(numberVal)) {
            return {
              result: false,
              error: '*Required'
            };
          }
        } else if (typeof value !== 'number') {
          return {
            result: false,
            error: '*Required'
          };
        }
      }

      if (!isValid(value, 'isDecimal')) {
        return { result: false, error: '*Invalid value type' };
      }

      if (isLessThanMinValue(value, schema.minValue)) {
        return { result: false, error: '*Invalid value' };
      }

      if (isGreaterThanMaxValue(value, schema.maxValue)) {
        return { result: false, error: '*Invalid value' };
      }
      return { result: true };
    case _dataTypes2.default.Date:
      if (schema.isRequired && (0, _util.isNullOrUndefined)(value)) {
        return {
          result: false,
          error: '*Required'
        };
      }
      return { result: true };
    case _dataTypes2.default.Array:
      if (schema.isRequired && ((0, _util.isNullOrUndefined)(value) || value.length === 0)) {
        return {
          result: false,
          error: '*Required'
        };
      }
      return { result: true };
    case _dataTypes2.default.Time:
      if (schema.isRequired && (0, _util.isNullOrUndefined)(value)) {
        return {
          result: false,
          error: '*Required'
        };
      }
      return { result: true };
    default:
      return { result: true };
  }
}

function validateSingleField(name, value, schema) {
  var fieldValue = value;
  if ((0, _util.isNullOrUndefined)(schema)) {
    return null;
  }

  if ((0, _util.isNullOrUndefined)(value) && (0, _util.isDefined)(schema.defaultValue)) {
    fieldValue = schema.defaultValue;
  }

  // If the field is array type than initialize it to an empty Array
  // if value is null or undefined.
  if ((0, _lodash.toUpper)(schema.fieldType) === _dataTypes2.default.Array && (0, _util.isNullOrUndefined)(fieldValue)) {
    fieldValue = [];
  }

  var validationResult = validateDataAgainstSchema(name, fieldValue, schema);

  // Apply decimal places to the number
  if (validationResult.result && (0, _lodash.toUpper)(schema.fieldType) === _dataTypes2.default.Number) {
    if (typeof fieldValue === 'string' && !(0, _lodash.isEmpty)(fieldValue) && (0, _lodash.isFinite)((0, _lodash.toNumber)(fieldValue))) {
      fieldValue = _util2.default.fixDouble((0, _lodash.toNumber)(fieldValue), schema.decimalPlaces);
    }
  }
  return { value: fieldValue, validationResult: _extends({}, validationResult) };
}

function validateObject(data, schema) {
  var outputData = {};
  (0, _lodash.forOwn)(schema, function (value, key) {
    var retValue = validateSingleField(key, data[key], value);
    if ((0, _util.isDefined)(retValue)) {
      outputData[key] = retValue;
    }
  });

  return outputData;
}